@@ -34,6 +34,26 @@ enum {
 };
 
 struct iscsi_conn;
+struct iscsi_ev_context;
+
+/*
+ * When handling async events, the initiator may not be able to
+ * handle the event in the same context, so this allows the interface
+ * code to call into the initiator to shedule handling.
+ */
+struct iscsi_ipc_ev_clbk {
+	void (*create_session) (uint32_t host_no, uint32_t sid);
+	void (*destroy_session) (uint32_t host_no, uint32_t sid);
+
+	struct iscsi_ev_context *(*get_ev_context) (struct iscsi_conn *conn,
+						    int ev_size);
+	void (*put_ev_context) (struct iscsi_ev_context *ev_context);
+	int (*sched_ev_context) (struct iscsi_ev_context *ev_context,
+				 struct iscsi_conn *conn,
+				 unsigned long tmo, int event);
+};
+
+extern void ipc_register_ev_callback(struct iscsi_ipc_ev_clbk *ipc_ev_clbk);
 
 /**
  * struct iscsi_ipc - Open-iSCSI Interface for Kernel IPC
@@ -109,6 +129,22 @@ struct iscsi_ipc {
 	int (*recv_pdu_begin) (struct iscsi_conn *conn);
 
 	int (*recv_pdu_end) (struct iscsi_conn *conn);
+
+	int (*set_net_config) (uint64_t transport_handle, uint32_t host_no,
+			       struct iovec *iovs, uint32_t param_count);
+
+	int (*recv_conn_state) (struct iscsi_conn *conn, uint32_t *state);
+
+	int (*exec_ping) (uint64_t transport_handle, uint32_t host_no,
+			  struct sockaddr *addr, uint32_t iface_num,
+			  uint32_t iface_type, uint32_t size, uint32_t *status);
+
+	int (*get_chap) (uint64_t transport_handle, uint32_t host_no,
+			 uint16_t chap_tbl_idx, uint32_t num_entries,
+			 char *chap_buf, uint32_t *valid_chap_entries);
+
+	int (*delete_chap) (uint64_t transport_handle, uint32_t host_no,
+			    uint16_t chap_tbl_idx);
 };
 
 #endif /* ISCSI_IPC_H */
