@@ -27,11 +27,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <poll.h>
+#include <errno.h>
 #include <sys/param.h>
 
 #include "initiator.h"
 #include "transport.h"
 #include "log.h"
+#include "iscsi_timer.h"
 
 /* caller is assumed to be well-behaved and passing NUL terminated strings */
 int
@@ -1434,11 +1437,15 @@ int
 iscsi_login_rsp(iscsi_session_t *session, iscsi_login_context_t *c)
 {
 	iscsi_conn_t *conn = &session->conn[c->cid];
+	int err;
 
 	/* read the target's response into the same buffer */
-	if (!iscsi_io_recv_pdu(conn, &c->pdu, ISCSI_DIGEST_NONE, c->data,
-			    c->max_data_length, ISCSI_DIGEST_NONE,
-			    c->timeout)) {
+	err = iscsi_io_recv_pdu(conn, &c->pdu, ISCSI_DIGEST_NONE, c->data,
+			        c->max_data_length, ISCSI_DIGEST_NONE,
+			        c->timeout);
+	if (err == -EAGAIN) {
+		goto done;
+	} else if (err < 0) {
 		/*
 		 * FIXME: caller might want us to distinguish I/O
 		 * error and timeout. Might want to switch portals on
@@ -1449,6 +1456,7 @@ iscsi_login_rsp(iscsi_session_t *session, iscsi_login_context_t *c)
 		goto done;
 	}
 
+	err = -EIO;
 	c->received_pdu = 1;
 
 	/* check the PDU response type */
@@ -1490,7 +1498,7 @@ iscsi_login_rsp(iscsi_session_t *session, iscsi_login_context_t *c)
 		if (c->ret == LOGIN_OK)
 			c->ret = LOGIN_FAILED;
 	}
-	return 1;
+	return err;
 }
 
 /**
@@ -1514,7 +1522,9 @@ iscsi_login(iscsi_session_t *session, int cid, char *buffer, size_t bufsize,
 {
 	iscsi_conn_t *conn = &session->conn[cid];
 	iscsi_login_context_t *c = &conn->login_context;
-	int ret;
+	struct timeval connection_timer;
+	struct pollfd pfd;
+	int ret, timeout;
 
 	/*
 	 * assume iscsi_login is only called from discovery, so it is
@@ -1532,15 +1542,63 @@ iscsi_login(iscsi_session_t *session, int cid, char *buffer, size_t bufsize,
 	do {
 		if (iscsi_login_req(session, c))
 			return c->ret;
-		ret = iscsi_login_rsp(session, c);
 
-		if (status_class)
-			*status_class = c->status_class;
-		if (status_detail)
-			*status_detail = c->status_detail;
+		/*
+		 * TODO: merge the poll and req/rsp code with the discovery
+		 * poll and text req/rsp.
+		 */
+		iscsi_timer_set(&connection_timer,
+				session->conn[0].active_timeout);
+		timeout = iscsi_timer_msecs_until(&connection_timer);
 
-		if (ret)
+		memset(&pfd, 0, sizeof (pfd));
+		pfd.fd = conn->socket_fd;
+		pfd.events = POLLIN | POLLPRI;
+
+repoll:
+		pfd.revents = 0;
+		ret = poll(&pfd, 1, timeout);
+		log_debug(7, "%s: Poll return %d\n", __FUNCTION__, ret);
+		if (iscsi_timer_expired(&connection_timer)) {
+			log_warning("Login response timeout. Waited %d "
+				    "seconds and did not get reponse PDU.\n",
+				    session->conn[0].active_timeout);
+			c->ret = LOGIN_FAILED;
 			return c->ret;
+		}
+
+		if (ret > 0) {
+			if (pfd.revents & (POLLIN | POLLPRI)) {
+				ret = iscsi_login_rsp(session, c);
+				if (ret ==  -EAGAIN)
+					goto repoll;
+
+				if (status_class)
+					*status_class = c->status_class;
+				if (status_detail)
+					*status_detail = c->status_detail;
+
+				if (ret)
+					return c->ret;
+			} else if (pfd.revents & POLLHUP) {
+				log_warning("Login POLLHUP");
+				c->ret = LOGIN_FAILED;
+				return c->ret;
+			} else if (pfd.revents & POLLNVAL) {
+				log_warning("Login POLLNVAL");
+				c->ret = LOGIN_IO_ERROR;
+				return c->ret;
+			} else if (pfd.revents & POLLERR) {
+				log_warning("Login POLLERR");
+				c->ret = LOGIN_IO_ERROR;
+				return c->ret;
+			}
+
+		} else if (ret < 0) {
+			log_error("Login poll error.\n");
+			c->ret = LOGIN_FAILED;
+			return c->ret;
+		}
 	} while (conn->current_stage != ISCSI_FULL_FEATURE_PHASE);
 
 	c->ret = LOGIN_OK;
