@@ -21,26 +21,21 @@
 #ifndef STRINGS_H
 #define STRINGS_H
 
-struct string_buffer {
+struct str_buffer {
 	size_t allocated_length;
 	size_t data_length;	/* not including the trailing NUL */
 	char *buffer;
 };
 
-extern int init_string_buffer(struct string_buffer *s,
-			      size_t initial_allocation);
-extern struct string_buffer *alloc_string_buffer(size_t initial_allocation);
-extern void free_string_buffer(struct string_buffer *s);
+extern int str_init_buffer(struct str_buffer *s, size_t initial_allocation);
+extern struct str_buffer *str_alloc_buffer(size_t initial_allocation);
+extern void str_free_buffer(struct str_buffer *s);
 
-extern void enlarge_data(struct string_buffer *s, int length);
-extern void remove_initial(struct string_buffer *s, int length);
-extern void truncate_buffer(struct string_buffer *s, size_t length);
-extern int append_string(struct string_buffer *s, const char *str);
-extern int append_sprintf(struct string_buffer *s, const char *format, ...);
-extern int adjoin_string(struct string_buffer *s, const char *str);
-extern char *buffer_data(struct string_buffer *s);
-extern size_t data_length(struct string_buffer *s);
-extern size_t unused_length(struct string_buffer *s);
-extern void write_buffer(struct string_buffer *s, int fd);
+extern int str_enlarge_data(struct str_buffer *s, int length);
+extern void str_remove_initial(struct str_buffer *s, int length);
+extern void str_truncate_buffer(struct str_buffer *s, size_t length);
+extern char *str_buffer_data(struct str_buffer *s);
+extern size_t str_data_length(struct str_buffer *s);
+extern size_t str_unused_length(struct str_buffer *s);
 
 #endif /* STRINGS_H */
