@@ -48,5 +48,7 @@ struct iscsi_transport {
 };
 
 extern int set_transport_template(struct iscsi_transport *t);
+extern int transport_load_kmod(char *transport_name);
+extern int transport_probe_for_offload(void);
 
 #endif
